/*----------------------------------------------------------------------*\
 * xrgb.c  X11 Red, Green, Blue color mixer program  Version 1.1	*
 * /UCAR/NCAR/ATD/RDP/ G. Forrest Cook, Jon Corbet  11/20/90		*
 *								        *
 * $Id: xrgb.c,v 1.13 1994/05/26 18:10:11 cook Exp $		*
\*----------------------------------------------------------------------*/

/*

	(C) Copyright 1994
	University Corporation for Atmospheric Research

The X Consortium, and any party obtaining a copy of these files from
the X Consortium, directly or indirectly, is granted, free of charge, a
full and unrestricted irrevocable, world-wide, paid up, royalty-free,
nonexclusive right and license to deal in this software and
documentation files (the "Software"), including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons who receive
copies from any such party to do so.  This license includes without
limitation a license to do the foregoing actions under any patents of
the party supplying this software to the X Consortium.

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

*/

/*
 * Version History
 * 1.0	First released version
 * 1.1  Eliminated unnecessary globals
 */

# include <stdio.h>
# include <X11/Xatom.h>
# include <X11/Intrinsic.h>
# include <X11/StringDefs.h>
# include <X11/Xaw/Form.h>
# include <X11/Xaw/Scrollbar.h>
# include <X11/Xaw/Label.h>
# include <X11/Xaw/Command.h>
# include <X11/Xaw/Cardinals.h>

void cjump(), cscroll(), xrgb_exit(), hex_select();

/*
 * Widgets that need to be global
 */
Widget Top, Lr, Lg, Lb;

XColor Xc;

# define RED 0
# define GREEN 1
# define BLUE 2

int Red = RED;
int Green = GREEN;
int Blue = BLUE;

#ifdef DEBUG
static char *Colors[3] = { "red", "green", "blue" };
#endif

/*
 * X11 rgb color name string for select function
 */
char *Rgb_string;

static String fallback_resources[] =
{
	"*title:			xrgb  1.1",
	"*quit.width:			100",
	"*quit.height:			24",
	"*quit.label:			Quit",
	"*quit.foreground:		magenta",
	"*quit.background:		black",
	"*mix.foreground:		black",
	"*mix.width:			100",
	"*mix.height:			256",
	"*mix.label:			Mixed",
	"*Label.resize:			FALSE",
	"*Label.width:			30",
	"*Label.height:			24",
	"*Label.background:		black",
	"*Label.foreground:		yellow",
	"*red.background:		red",
	"*green.background:		green",
	"*blue.background:		blue",
	"*Scrollbar.foreground:		black",
	"*Scrollbar.height:		256",
	"*Scrollbar.width:		30",
	"*select.width:			208",
	"*select.height:		24",
	"*select.label:			Select",
	"*select.foreground:		cyan",
	"*select.background:		black",
	NULL
};

/*----------------------------------------------------------------------*\
 * define a black pixmap here.						*
\*----------------------------------------------------------------------*/

#define black_width 8
#define black_height 1
static unsigned char black_bits[] = { 0xff };

/*----------------------------------------------------------------------*/

main (argc, argv)
int argc;
char **argv;
{
	int n;
	Arg args[10];
        Pixmap thumb_pm;
	Widget w, form, label, sbr, sbg, sbb, lq, lh;
	XtAppContext actx;

/*
 * Set some defaults.
 */
	Xc.red = Xc.green = Xc.blue = 65535;
	Xc.flags = DoRed | DoGreen | DoBlue;
/*
 * Create an application shell, and a form to go inside it.
 * The second arg must differ from the fallback resource name.
 */
	Top = XtAppInitialize (&actx, "Xrgb", (XrmOptionDescList)NULL,ZERO,
		&argc, argv, fallback_resources, (ArgList)NULL, ZERO);

	form = XtCreateManagedWidget ("form", formWidgetClass, Top,NULL,ZERO);
/*
 * Create a select button that fills the X select buffer with an X
 * compatible RGB color string when pushed.
 */
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, NULL);		n++;
	XtSetArg (args[n], XtNfromVert, NULL);		n++;
	lh = XtCreateManagedWidget ("select", commandWidgetClass, form,args,n);
	XtAddCallback (lh, XtNcallback, hex_select, (void *)0);
/*
 * create 3 number-slider pairs
 */
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, NULL);		n++;
	XtSetArg (args[n], XtNfromVert, lh);		n++;
	Lr = XtCreateManagedWidget ("rval", labelWidgetClass, form,
			args, n);
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, NULL);		n++;
	XtSetArg (args[n], XtNfromVert, Lr);		n++;
	XtSetArg (args[n], XtNorientation, XtorientVertical); n++;
	w=sbr=XtCreateManagedWidget ("red", scrollbarWidgetClass,form,args,n);
	XtAddCallback (w, XtNscrollProc, cscroll, &Red);
	XtAddCallback (w, XtNjumpProc, cjump, &Red);

	n = 0;
	XtSetArg (args[n], XtNfromHoriz, Lr);		n++;
	XtSetArg (args[n], XtNfromVert, lh);		n++;
	Lg = XtCreateManagedWidget ("gval", labelWidgetClass, form, args, n);
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, w);		n++;
	XtSetArg (args[n], XtNfromVert, Lg);		n++;
	XtSetArg (args[n], XtNorientation, XtorientVertical); n++;
	w=sbg=XtCreateManagedWidget ("green",scrollbarWidgetClass,form,args,n);
	XtAddCallback (w, XtNscrollProc, cscroll, &Green);
	XtAddCallback (w, XtNjumpProc, cjump, &Green);

	n = 0;
	XtSetArg (args[n], XtNfromHoriz, Lg);		n++;
	XtSetArg (args[n], XtNfromVert, lh);		n++;
	Lb = XtCreateManagedWidget ("bval", labelWidgetClass, form, args, n);
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, w);		n++;
	XtSetArg (args[n], XtNfromVert, Lb);		n++;
	XtSetArg (args[n], XtNorientation, XtorientVertical); n++;
	w=sbb=XtCreateManagedWidget ("blue",scrollbarWidgetClass,form,args,n);
	XtAddCallback (w, XtNscrollProc, cscroll, &Blue);
	XtAddCallback (w, XtNjumpProc, cjump, &Blue);
/*
 * Create a quit button.
 */
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, Lb);		n++;
	XtSetArg (args[n], XtNfromVert, lh);		n++;
	lq = XtCreateManagedWidget ("quit", commandWidgetClass, form, args, n);
	XtAddCallback (lq, XtNcallback, xrgb_exit, (void *)0);
/*
 * Get a color cell.
 */
	XAllocColorCells (XtDisplay (Top), DefaultColormap (XtDisplay(Top), 0),
		False, NULL, 0, &Xc.pixel, 1);
/*
 * Create the label to show the color.
 */
	n = 0;
	XtSetArg (args[n], XtNfromHoriz, Lb);		n++;
	XtSetArg (args[n], XtNfromVert, lq);		n++;
	label = XtCreateManagedWidget ("mix", labelWidgetClass, form, args, n);
/*
 * Assign a color to the label.
 */
	n = 0;
	XtSetArg (args[n], XtNbackground, Xc.pixel);	n++;
	XtSetValues (label, args, n);
/*
 * Run the beast.
 */
	SetLabelNumber (Lr, 255);
	SetLabelNumber (Lg, 255);
	SetLabelNumber (Lb, 255);

	XtRealizeWidget (Top);
/*
 * Make a black bitmap for the scrollbar thumbs.
 */
        thumb_pm = XCreateBitmapFromData (XtDisplay (Top),
                XtWindow (sbr), black_bits, black_width, black_height);
        if (!thumb_pm)
                printf ("\nError, can't make thumb pixmap\n");

        XtSetArg (args[0], XtNthumb, thumb_pm);
        XtSetValues (sbr, args, 1);
        XtSetValues (sbg, args, 1);
        XtSetValues (sbb, args, 1);

	SetLabelColor ();
	XtAppMainLoop (actx);
}

/*----------------------------------------------------------------------*/

void
xrgb_exit (q, dummy1, dummy2)
Widget q;
int *dummy1;
int dummy2;
{
	exit (1);
}

/*----------------------------------------------------------------------*/

void
cscroll (sb, color, position)
Widget sb;
int *color;
int position;
{
	int newc;

	switch (*color)
	{
	   case RED:
		if (position > 0)
		{
			if (Xc.red < 65280) Xc.red += 256;
		}
		else if (Xc.red > 255) Xc.red -= 256;
		newc = Xc.red;
		SetLabelNumber (Lr, newc>>8);
		break;
	   case GREEN:
		if (position > 0)
		{
			if (Xc.green < 65280) Xc.green += 256;
		}
		else if (Xc.green > 255) Xc.green -= 256;
		newc = Xc.green;
		SetLabelNumber (Lg, newc>>8);
		break;
	   case BLUE:
		if (position > 0)
		{
			if (Xc.blue < 65280) Xc.blue += 256;
		}
		else if (Xc.blue > 255) Xc.blue -= 256;
		newc = Xc.blue;
		SetLabelNumber (Lb, newc>>8);
		break;
	}

        XawScrollbarSetThumb (sb, 1.0 - (float)newc/65535.0, -1.0);

	SetLabelColor ();

#ifdef DEBUG
	printf ("Scroll on %s, position = %d\n", Colors[color], position);
#endif
}

/*----------------------------------------------------------------------*/

void
cjump (sb, color, pct)
Widget sb;
int *color;
XtPointer pct;
{
	int pos = 65535 * (1 - (* (float *)pct));

#ifdef DEBUG
	printf ("Jump to %d on %s\n", pos, Colors[*color]);
#endif

	switch (*color)
	{
	   case RED:
	   	Xc.red = pos;
		SetLabelNumber (Lr, pos>>8);
		break;
	   case GREEN:
	   	Xc.green = pos;
		SetLabelNumber (Lg, pos>>8);
		break;
	   case BLUE:
	   	Xc.blue = pos;
		SetLabelNumber (Lb, pos>>8);
		break;
	}

	SetLabelColor ();
}

/*----------------------------------------------------------------------*/

SetLabelColor ()
/*
 * Set the background color in our label widget.
 */
{
	XStoreColor (XtDisplay(Top), DefaultColormap(XtDisplay(Top),0), &Xc);
}

/*----------------------------------------------------------------------*/

SetLabelNumber (wi, num)  /* put a decimal number into the label widget */
Widget wi;
int num;
{
	char ostr[10];
	int n = 0;
	Arg args[1];

	sprintf (ostr, "%02x", num);

	XtSetArg (args[n], XtNlabel, ostr);             n++;
	XtSetValues (wi, args, n);
}

/*----------------------------------------------------------------------*\
 * The following function was grabbed from xfontsel.		        *
 *                                                                      *
 * Copyright 1985, 1986, 1987, 1988, 1989 by the                        *
 * Massachusetts Institute of Technology                                *
\*----------------------------------------------------------------------*/

Boolean
ConvertSelection(w, selection, target, type, value, length, format)
    Widget w;
    Atom *selection, *target, *type;
    XtPointer *value;
    unsigned long *length;
    int *format;
{
	/* XmuConvertStandardSelection will use the second parameter only when
	 * converting to the target TIMESTAMP.  However, it will never be
	 * called upon to perform this conversion, because Xt will handle it
	 * internally.  CurrentTime will never be used.
	 */

	if (XmuConvertStandardSelection(w, CurrentTime, selection, target,
		type, (caddr_t *) value, length, format))
		return True;

	if (*target == XA_STRING)
	{
		*type = XA_STRING;
		*value = Rgb_string;
		*length = strlen(*value);
		*format = 8;
		return True;
	}
	else return False;
}

/*----------------------------------------------------------------------*/

void
hex_select (w)
Widget w;
{
	Time time = XtLastTimestampProcessed(XtDisplay(w));

/*
 * Grab some memory for the selection string, it gets freed externally.
 * Stuff a color value into the string.
 */
	Rgb_string = XtMalloc (20);

	sprintf (Rgb_string, "rgb:%02x/%02x/%02x",
		Xc.red>>8, Xc.green>>8, Xc.blue>>8);

	XtOwnSelection (w, XA_PRIMARY, time, ConvertSelection, NULL, NULL);
}

/*----------------------------------------------------------------------*/
